// strhelp.h - template base string classes for Win32(R) Platform
// Copyright (C) 2000 by Tietew.
// 
// You can reuse and/or redistribute this code without any permission.
//
#pragma once
#include <algorithm>

#pragma pack(push, 1)

template<class T> struct CharTraits;
template<> struct CharTraits<char>
{
	typedef size_t size_type;
	typedef ptrdiff_t difference_type;
	typedef char char_type;
	typedef char *pointer;
	typedef const char *const_pointer;
	typedef char& reference;
	typedef const char& const_reference;
	static void Copy(pointer _Dest, const_pointer _Src, size_type _Size)
		{ memcpy(_Dest, _Src, _Size * sizeof(char)); }
	static void Fill(pointer _Dest, const char_type& _Ch, size_type _Size)
		{ memset(_Dest, _Ch, _Size); }
	static size_type Length(const_pointer _Str)
		{ return strlen(_Str); }
	static bool Equal(const_pointer _L, const_pointer _R)
		{ return strcmp(_L, _R) == 0; }
	static bool Less(const_pointer _L, const_pointer _R)
		{ return strcmp(_L, _R) < 0; }
	static void UpperCase(pointer _Ch)
		{ CharUpperA(_Ch); }
	static void LowerCase(pointer _Ch)
		{ CharLowerA(_Ch); }
	static bool IsSpace(const_reference _Ch)
		{ return (unsigned char)_Ch <= (unsigned char)' '; }
};
template<> struct CharTraits<wchar_t>
{
	typedef size_t size_type;
	typedef ptrdiff_t difference_type;
	typedef wchar_t char_type;
	typedef wchar_t *pointer;
	typedef const wchar_t *const_pointer;
	typedef wchar_t& reference;
	typedef const wchar_t& const_reference;
	static void Copy(pointer _Dest, const_pointer _Src, size_type _Size)
		{ memcpy(_Dest, _Src, _Size * sizeof(wchar_t)); }
	static void Fill(pointer _Dest, const char_type& _Ch, size_type _Size)
		{ std::fill(_Dest, _Dest + _Size, _Ch); }
	static size_type Length(const_pointer _Str)
		{ return wcslen(_Str); }
	static bool Equal(const_pointer _L, const_pointer _R)
		{ return wcscmp(_L, _R) == 0; }
	static bool Less(const_pointer _L, const_pointer _R)
		{ return wcscmp(_L, _R) < 0; }
	static void UpperCase(pointer _Ch)
		{ CharUpperW(_Ch); }
	static void LowerCase(pointer _Ch)
		{ CharLowerW(_Ch); }
	static bool IsSpace(const_reference _Ch)
		{ return _Ch <= L' '; }
};

template<class TL, class TR, class Traits>
	class StringAppender
{
	TL L;
	TR R;
public:
	StringAppender(const TL& _L, const TR& _R)
		: L(_L), R(_R) { }
	typename Traits::size_type length() const
		{ return L.length() + R.length(); }
	typename Traits::pointer copy(typename Traits::pointer _Dest) const
		{ return R.copy(L.copy(_Dest)); }
};
template<class Traits>
	class StringWrapper
{
	typename Traits::const_pointer Str;
	typename Traits::size_type Len;
public:
	StringWrapper(typename Traits::const_pointer _Str)
		: Str(_Str) { Len = Traits::Length(_Str); }
	typename Traits::size_type length() const
		{ return Len; }
	typename Traits::pointer copy(typename Traits::pointer _Dest) const
		{ Traits::Copy(_Dest, Str, Len);
		return _Dest + Len; }
};
template<class Traits>
	class CharWrapper
{
	typename Traits::char_type Ch;
public:
	CharWrapper(typename Traits::const_reference _Ch)
		: Ch(_Ch) { }
	typename Traits::size_type length() const
		{ return 1; }
	typename Traits::pointer copy(typename Traits::pointer _Dest) const
		{ *_Dest = Ch;
		return _Dest + 1; }
};

template<class T, class Traits = CharTraits<T> >
	class StringBase
{
public:
	typedef StringBase<T, Traits> Myt;
	typedef Traits traits_type;
	typedef typename Traits::char_type value_type;
	typedef typename Traits::size_type size_type;
	typedef typename Traits::difference_type difference_type;
	typedef typename Traits::const_pointer iterator;
	typedef typename Traits::const_pointer const_iterator;
	typedef typename Traits::pointer Tptr;
	typedef typename Traits::const_pointer Ctptr;
	typedef typename Traits::const_reference reference;
	typedef typename Traits::const_reference const_reference;
	typedef std::reverse_iterator<iterator, value_type, reference, Ctptr,
		difference_type> reverse_iterator;
	typedef std::reverse_iterator<const_iterator, value_type,
		const_reference, Ctptr, difference_type> const_reverse_iterator;
	class _AtProxy {
		friend class StringBase<T, Traits>;
		Myt& Str;
		difference_type Idx;
		_AtProxy(Myt& _Str, difference_type _Idx) : Str(_Str), Idx(_Idx)
			{ }
	public:
		operator value_type() const
			{ return *(Str.Str + Idx); }
		reference operator=(const_reference _Ch)
			{ Str._Dup();
			*(Str.Str + Idx) = _Ch;
			return *(Str.Str + Idx); }
		Ctptr operator&()
			{ return Str.Str + Idx; }
	};
	friend class _AtProxy;
	class _AppendStub {
		const Myt& Str;
	public:
		_AppendStub(const _AppendStub& _Stub)
			: Str(_Stub.Str) { }
		explicit _AppendStub(const Myt& _Str)
			: Str(_Str) { }
		size_type length() const
			{ return Str.length(); }
		Tptr copy(Tptr _Dest) const
			{ return Str.copy(_Dest); }
	};
	struct _NilString { };
	static const _NilString Nil;
	static const _NilString NilString()
		{ return Nil; }
	StringBase()
		{ Str = _Nil.Str; }
	StringBase(const_reference _Ch)
		{ _Buy(1);
		*Str = _Ch;
		*(Str + 1) = 0; }
	StringBase(Ctptr _Str)
		{ *_Str ? _Buy(_Str, Traits::Length(_Str)) : Str = _Nil.Str; }
	StringBase(const Myt& _Str)
		{ Str = _Str.Str;
		_Addref(); }
	StringBase(const _NilString& _Str)
		{ Str = _Nil.Str; }
	template<class FwdIt> StringBase(FwdIt _F, FwdIt _L)
		{ if(_F == _L)
			Str = _Nil.Str;
		else
			{ _Buy(std::distance(_F, _L));
			*std::copy(_F, _L, Str) = 0; } }
	template<> StringBase(Tptr _F, Tptr _L)
		{ if(_F == _L)
			Str = _Nil.Str;
		else
			{ _Buy(_F, _L - _F);
			*(Str + length()) = 0; } }
	template<> StringBase(Ctptr _F, Ctptr _L)
		{ if(_F == _L)
			Str = _Nil.Str;
		else
			{ _Buy(_F, _L - _F);
			*(Str + length()) = 0; } }
	template<class TL, class TR>
		StringBase(const StringAppender<TL, TR, Traits>& _App)
			{ size_type _Len = _App.length();
			if(_Len == 0)
				Str = _Nil.Str;
			else
				{ _Buy(_Len);
				*_App.copy(Str) = 0; } }
	explicit StringBase(size_type _Len, Ctptr _Str)
		{ _Len == 0 ? Str = _Nil.Str : _Buy(_Str, _Len); }
	explicit StringBase(const_reference _Ch, size_type _Num)
		{ if(_Num == 0)
			Str = _Nil.Str;
		else
			{ _Buy(_Num);
			Traits::Fill(Str, _Ch, _Num);
			*(Str + _Num) = 0; } }
	explicit StringBase(Ctptr _Str, size_type _Num)
		{ if(_Num == 0 || !*_Str)
			Str = _Nil.Str;
		else if(_Num == 1)
			_Buy(_Str, Traits::Length(_Str));
		else
			{ size_type _Len = Traits::Length(_Str);
			_Buy(_Len * _Num);
			for(size_type i = 0; i < _Num; ++i)
				Trais::Copy(Str + _Len * i, _Str, _Len);
			*(Str + _Len * _Num) = 0; } }
	explicit StringBase(const Myt& _Str, size_type _Num)
		{ if(_Num == 0 || _Str.Str == _Nil.Str)
			Str = _Nil.Str;
		else if(_Num == 1)
			{ Str = _Str.Str;
			_Addref(); }
		else
			{ _Buy(_Str.length() * _Num);
			for(size_type i = 0; i < _Num; ++i)
				Traits::Copy(Str + _Str.length() * i, _Str.Str, _Str.length());
			*(Str + _Str.length() * _Num) = 0; } }
	explicit StringBase(size_type _Len)
		{ _Buy(_Len);
		*Str = 0; }
	~StringBase()
		{ _Release(); }
	operator Ctptr() const
		{ return Str; }
	_AtProxy operator[](difference_type _Idx)
		{ return _AtProxy(*this, _Idx); }
	const_reference operator[](difference_type _Idx) const
		{ return *(Str + _Idx); }
	const_iterator begin() const
		{ return iterator(Str); }
	const_iterator end() const
		{ return iterator(Str + length()); }
	const_reverse_iterator rbegin() const
		{ return const_reverse_iterator(end()); }
	const_reverse_iterator rend() const
		{ return const_reverse_iterator(begin()); }
	_AtProxy front()
		{ return _AtProxy(*this, 0); }
	const_reference front() const
		{ return *begin(); }
	_AtProxy back()
		{ return _AtProxy(*this, length() - 1); }
	const_reference back() const
		{ return *(end() - 1); }
	size_type length() const
		{ return _GetData()->Len; }
	size_type size() const
		{ return _GetData()->Len; }
	Myt& operator=(const_reference _Ch)
		{ _Dup(1, false);
		*Str = _Ch;
		*(Str + 1) = 0;
		return *this; }
	Myt& operator=(Ctptr _Str)
		{ if(*_Str)
			{ size_type _Len = Traits::Length(_Str);
			_Dup(_Len, false);
			Traits::Copy(Str, _Str, _Len + 1); }
		else
			{ _Release();
			Str = _Nil.Str; }
		return *this; }
	Myt& operator=(const Myt& _Str)
		{ if(this != &_Str)
			{ _Release();
			Str = _Str.Str;
			_Addref(); }
		return *this; }
	const _NilString& operator=(const _NilString& _NilStr)
		{ _Release();
		Str = _Nil.Str;
		return _NilStr; }
	Myt& assign(const_reference _Ch)
		{ return *this = _Ch; }
	Myt& assign(Ctptr _Str)
		{ return *this = _Str; }
	Myt& assign(const Myt& _Str)
		{ return *this = _Str; }
	const _NilString& assign(const _NilString& _NilStr)
		{ return *this = _NilStr; }
	template<class TL, class TR>
		Myt& assign(const StringAppender<TL, TR, Traits>& _App)
		{ return *this = _App; }
	template<class FwdIt>
		Myt& assign(FwdIt _F, FwdIt _L)
			{ if(_F == _L)
				{ _Release();
				Str = _Nil.Str; }
			else
				{ _Dup(std::distance(_F, _L), false);
				*std::copy(_F, _L, Str) = 0; }
			return *this; }
	template<> Myt& assign(Ctptr _F, Ctptr _L)
		{ if(_F == _L)
			{ _Release();
			Str = _Nil.Str; }
		else
			{ _Dup(_L - _F, false);
			Traits::Copy(Str, _F, _L - _F);
			*(Str + length()) = 0; }
		return *this; }
	template<> Myt& assign(Tptr _F, Tptr _L)
		{ return assign(static_cast<Ctptr>(_F), static_cast<Ctptr>(_L)); }
	friend inline bool operator==(const Myt& _L, const Myt& _R)
		{ return _L.Str == _R.Str || (_L.length() == _R.length() && Traits::Equal(_L.Str, _R.Str)); }
	friend inline bool operator==(const Myt& _L, Ctptr _R)
		{ return !*_R && _L.Str == _Nil.Str || Traits::Equal(_L.Str, _R); }
	friend inline bool operator==(const Myt& _L, const_reference _R)
		{ return *_L.Str == _R && *(_L.Str + 1) == 0; }
	friend inline bool operator==(const Myt& _L, const _NilString& _R)
		{ return _L.Str == _Nil.Str; }
	friend inline bool operator==(Ctptr _L, const Myt& _R)
		{ return _R == _L; }
	friend inline bool operator==(const_reference _L, const Myt& _R)
		{ return _R == _L; }
	friend inline bool operator==(const _NilString& _L, const Myt& _R)
		{ return _R == _L; }
	friend inline bool operator!=(const Myt& _L, const Myt& _R)
		{ return !(_L == _R); }
	friend inline bool operator!=(const Myt& _L, Ctptr _R)
		{ return !(_L == _R); }
	friend inline bool operator!=(const Myt& _L, const_reference _R)
		{ return !(_L == _R); }
	friend inline bool operator!=(const Myt& _L, const _NilString& _R)
		{ return !(_L == _R); }
	friend inline bool operator!=(Ctptr _L, const Myt& _R)
		{ return !(_R == _L); }
	friend inline bool operator!=(const_reference _L, const Myt& _R)
		{ return !(_R == _L); }
	friend inline bool operator!=(const _NilString& _L, const Myt& _R)
		{ return !(_R == _L); }
	bool operator<(const Myt& _Str) const
		{ return Traits::Less(Str, _Str.Str); }
	bool operator>(const Myt& _Str) const
		{ return _Str < *this; }
	bool operator<=(const Myt& _Str) const
		{ return !(_Str < *this); }
	bool operator>=(const Myt& _Str) const
		{ return !(*this < _Str); }
	operator bool() const
		{ return *Str ? true : false; }
	bool operator!() const
		{ return !operator bool(); }
	Myt& operator+=(const_reference _Ch)
		{ size_type _Len = _Dup(length() + 1);
		*(Str + _Len) = _Ch;
		*(Str + _Len + 1) = 0;
		return *this; }
	Myt& operator+=(Ctptr _Str)
		{ if(*_Str)
			{ size_type _Len = Traits::Length(_Str);
			size_type _Len0 = _Dup(length() + _Len);
			Traits::Copy(Str + _Len0, _Str, _Len + 1); }
		return *this; }
	Myt& operator+=(const Myt& _Str)
		{ if(_Str.Str != _Nil.Str)
			{ size_type _Len0 = _Dup(length() + _Str.length());
			Traits::Copy(Str + _Len0, _Str.Str, _Str.length() + 1); }
		return *this; }
	template<class TL, class TR>
		Myt& operator+=(const StringAppender<TL, TR, Traits>& _App)
		{ size_type _Len = _App.length();
		if(_Len != 0)
			{ size_type _Len0 = _Dup(length() + _Len);
			*(_App.copy(Str + _Len0)) = 0; }
		return *this; }
	Myt& append(const_reference _Ch)
		{ return *this += _Ch; }
	Myt& append(Ctptr _Str)
		{ return *this += _Str; }
	Myt& append(const Myt& _Str)
		{ return *this += _Str; }
	template<class TL, class TR>
		Myt& append(const StringAppender<TL, TR, Traits>& _App)
		{ return *this += _App; }
	template<class FwdIt>
		Myt& append(FwdIt _F, FwdIt _L)
		{ if(_F != _L)
			{ size_type _Len0 = _Dup(length() + std::distance(_F, _L));
			*std::copy(_F, _L, Str + _Len0) = 0; }
		return *this; }
	template<>
		Myt& append(Ctptr _F, Ctptr _L)
		{ if(_F != _L)
			{ size_type _Len0 = _Dup(length() + _L - _F);
			Traits::Copy(Str + Len0, _F, _L - _F);
			*(Str + length()) = 0; }
		return *this; }
	Myt& operator*=(size_type _Num)
		{ return *this = *this * _Num; }
	friend inline Myt operator*(const Myt& _Str, size_type _Num)
		{ return Myt(_Str, _Num); }
	Tptr copy(Tptr _Ptr) const
		{ Traits::Copy(_Ptr, Str, length());
		return _Ptr + length(); }
	Myt substr(difference_type _F) const
		{ return Myt(Str + _F, Str + length()); }
	Myt substr(difference_type _F, difference_type _L) const
		{ return Myt(Str + _F, Str + _L); }
	Myt to_upper() const
		{ Myt result(*this);
		Traits::UpperCase(result.Alloc());
		return result.Fix(); }
	Myt to_lower() const
		{ Myt result(*this);
		Traits::LowerCase(result.Alloc());
		return result.Fix(); }
	Myt& trim()
		{ if(Str != _Nil.Str)
			{ Ctptr _F = Str, _L = Str + length() - 1;
			while(*_F && Traits::IsSpace(*_F))
				++_F;
			while(_L > _F && Traits::IsSpace(*_L))
				--_L;
			assign(_F, ++_L); }
		return *this; }
	Myt trimed() const
		{ if(Str == _Nil.Str)
			return Nil;
		Ctptr _F = Str, _L = Str + length() - 1;
		while(*_F && Traits::IsSpace(*_F))
			++_F;
		while(_L > _F && Traits::IsSpace(*_L))
			--_L;
		return Myt(_F, ++_L); }
	Myt& chop()
		{ if(Str != _Nil.Str)
			if(length() == 1)
				{ _Release();
				Str = _Nil.Str; }
			else
				{ _Dup(length() - 1);
				*(Str + length()) = 0; }
		return *this; }
	Myt chopped() const
		{ return length() <= 1 ? Nil : Myt(Str, Str + lenght() - 1); }
	Myt token()
		{ Ctptr _F = Str, _L, _LL = Str + length();
		while(_F != _LL && Traits::IsSpace(*_F))
			++_F;
		for(_L = _F; _L != _LL && !Traits::IsSpace(*_L); ++_L)
			;
		Myt _Tk(_F, _L);
		while(_L != _LL && Traits::IsSpace(*_L))
			++_L;
		assign(_L, (Ctptr)Str + length());
		return _Tk; }
	template<class Pred>
		Myt token(Pred _Pr)
		{ Ctptr _F = Str, _L, _LL = Str + length();
		while(_F != _LL && _Pr(*_F))
			++_F;
		for(_L = _F; _L != _LL && !_Pr(*_L); ++_L)
			;
		Myt _Tk(_F, _L);
		while(_L != _LL && _Pr(*_L))
			++_L;
		assign(_L, (Ctptr)Str + length());
		return _Tk; }
	template<class OutIt>
		OutIt put_token(OutIt _X) const
		{ Ctptr _F = Str, _L, _LL = Str + length();
		while(1)
			{ while(_F != _LL && Traits::IsSpace(*_F))
				++_F;
			if(_F == _LL)
				break;
			for(_L = _F; _L != _LL && !Traits::IsSpace(*_L); ++_L)
				;
			*_X++ = Myt(_F, _L); }
		return _X; }
	template<class OutIt, class Pred>
		OutIt put_token(OutIt _X, Pred _Pr) const
		{ Ctptr _F = Str, _L, _LL = Str + length();
		while(1)
			{ while(_F != _LL && _Pr(*_F))
				++_F;
			if(_F == _LL)
				break;
			for(_L = _F; _L != _LL && !_Pr(*_L); ++_L)
				;
			*_X++ = Myt(_F, _L); }
		return _X; }
	void swap(Myt& _Str)
		{ std::swap(Str, _Str.Str); }
	friend inline void swap(Myt& _L, Myt& _R)
		{ _L.swap(_R); }
	Tptr Alloc()
		{ _Dup();
		return Str; }
	Tptr Alloc(size_type _Len)
		{ _Dup(_Len);
		return Str; }
	Myt& Fix()
		{ return Fix(Traits::Length(Str)); }
	Myt& Fix(size_type _Len)
		{ if(_Len)
			{ *(Str + _Len) = 0;
			_GetData()->Len = _Len; }
		else
			{ _Release();
			Str = _Nil.Str; }
		return *this; }
	static inline StringAppender<StringWrapper<Traits>, CharWrapper<Traits>, Traits>
		concat(Ctptr _L, const_reference _R)
		{ return StringAppender<StringWrapper<Traits>, CharWrapper<Traits>,
			Traits>(StringWrapper<Traits>(_L), CharWrapper<Traits>(_R)); }
	static inline StringAppender<CharWrapper<Traits>, StringWrapper<Traits>, Traits>
		concat(const_reference _L, Ctptr _R)
		{ return StringAppender<CharWrapper<Traits>, StringWrapper<Traits>,
			Traits>(CharWrapper<Traits>(_L), StringWrapper<Traits>(_R)); }
	static inline StringAppender<StringWrapper<Traits>, StringWrapper<Traits>, Traits>
		concat(Ctptr _L, Ctptr _R)
		{ return StringAppender<StringWrapper<Traits>, StringWrapper<Traits>,
			Traits>(StringWrapper<Traits>(_L), StringWrapper<Traits>(_R)); }
	static inline Myt parse(long _Num)
		{ value_type _Buff[12];
		Tptr _Ptr = _Buff + 12;
		bool _Neg = _Num < 0;
		if(_Neg)
			_Num = -_Num;
		do
			*--_Ptr = (_Num % 10) + value_type('0'), _Num /= 10;
		while(_Num);
		if(_Neg)
			*--_Ptr = value_type('-');
		return Myt(_Ptr, _Buff + 12); }
	static inline Myt parse(int _Num)
		{ return parse((long)_Num); }
	static inline Myt parse(short _Num)
		{ return parse((long)_Num); }
	static inline Myt parse(unsigned long _Num)
		{ value_type _Buff[12];
		Tptr _Ptr = _Buff + 12;
		do
			*--_Ptr = (_Num % 10) + value_type('0'), _Num /= 10;
		while(_Num);
		return Myt(_Ptr, _Buff + 12); }
	static inline Myt parse(unsigned int _Num)
		{ return parse((unsigned long)_Num); }
	static inline Myt parse(unsigned short _Num)
		{ return parse((unsigned long)_Num); }

protected:
	struct Data {
		long Refcnt;
		size_type Cap;
		size_type Len;
		value_type Str[1];
	};
//	StringBase(Ctptr _L, size_type _LLen, const_reference _R)
//		{ _Buy(_LLen + 1);
//		Traits::Copy(Str, _L, _LLen);
//		*(Str + _LLen) = _R;
//		*(Str + _LLen + 1) = 0; }
//	StringBase(const_reference _L, Ctptr _R, size_type _RLen)
//		{ _Buy(_RLen + 1);
//		*Str = _L;
//		Traits::Copy(Str + 1, _R, _RLen + 1); }
//	StringBase(Ctptr _L, size_type _LLen, Ctptr _R, size_type _RLen)
//		{ _Buy(_LLen + _RLen);
//		Traits::Copy(Str, _L, _LLen);
//		Traits::Copy(Str + _LLen, _R, _RLen + 1); }
	Data *_GetData()
		{ return reinterpret_cast<Data*>(Str + 1) - 1; }
	const Data *_GetData() const
		{ return reinterpret_cast<const Data*>(Str + 1) - 1; }
	void _Buy(size_type _Len, size_type _Cap)
		{ Data *_Data = reinterpret_cast<Data*>(new char[_Cap * sizeof(value_type) + sizeof(Data)]);
		_Data->Cap = _Cap;
		_Data->Len = _Len;
		_Data->Refcnt = 1;
		Str = _Data->Str; }
	void _Buy(size_type _Len)
		{ _Buy(_Len, _Len); }
	void _Buy(Ctptr _Str, size_type _Len, size_type _Cap)
		{ _Buy(_Len, _Cap);
		Traits::Copy(Str, _Str, _Len + 1); }
	void _Buy(Ctptr _Str, size_type _Len)
		{ _Buy(_Str, _Len, _Len); }
	void _Free(Data *_Data)
		{ delete[] reinterpret_cast<char*>(_Data); }
	void _Addref()
		{ InterlockedIncrement(&_GetData()->Refcnt); }
	void _Release()
		{ if(Str != _Nil.Str && !InterlockedDecrement(&_GetData()->Refcnt))
			_Free(_GetData()); }
	void _Dup()
		{ if(Str != _Nil.Str && _GetData()->Refcnt != 1)
			{ Data *_Data = _GetData();
			_Buy(_Data->Str, _Data->Len, _Data->Len);
			InterlockedDecrement(&_Data->Refcnt); } }
	size_type _Dup(size_type _Len, bool _Cp = true)
		{ if(Str == _Nil.Str && _Len == 0)
			return 0;
		Data *_Data = _GetData();
		size_type _Len0 = _Data->Len;
		if(_Data->Refcnt == 1 && _Len <= _Data->Cap)
			{ _Data->Len = _Len;
			return _Len0; }
		if(_Len <= _Data->Cap)
			_Buy(_Len, _Data->Cap);
		else if(_Len <= _Data->Cap * 2)
			_Buy(_Len, _Data->Cap * 2);
		else
			_Buy(_Len, _Data->Cap + _Len);
		if(_Cp)
			Traits::Copy(Str, _Data->Str, (_Len0 < _Len ? _Len0 : _Len) + 1);
		if(_Data->Str != _Nil.Str && !InterlockedDecrement(&_Data->Refcnt))
			_Free(_Data);
		return _Len0; }
	static Data _Nil;
	Tptr Str;
};
template<class T, class Traits>
	StringBase<T, Traits>::Data StringBase<T, Traits>::_Nil = { 0, 0, 0, 0 };
template<class T, class Traits>
	const StringBase<T, Traits>::_NilString StringBase<T, Traits>::Nil;

template<class T, class Traits> inline
	StringAppender<StringBase<T, Traits>::_AppendStub, CharWrapper<Traits>, Traits>
	operator+(const StringBase<T, Traits>& _L, typename Traits::const_reference _R)
		{ return StringAppender<StringBase<T, Traits>::_AppendStub, CharWrapper<Traits>,
			Traits>(StringBase<T, Traits>::_AppendStub(_L), CharWrapper<Traits>(_R)); }
template<class T, class Traits> inline
	StringAppender<StringBase<T, Traits>::_AppendStub, StringWrapper<Traits>, Traits>
	operator+(const StringBase<T, Traits>& _L, typename Traits::const_pointer _R)
		{ return StringAppender<StringBase<T, Traits>::_AppendStub, StringWrapper<Traits>,
			Traits>(StringBase<T, Traits>::_AppendStub(_L), StringWrapper<Traits>(_R)); }
template<class T, class Traits> inline
	StringAppender<CharWrapper<Traits>, StringBase<T, Traits>::_AppendStub, Traits>
	operator+(typename Traits::const_reference _L, const StringBase<T, Traits>& _R)
		{ return StringAppender<CharWrapper<Traits>, StringBase<T, Traits>::_AppendStub,
			Traits>(CharWrapper<Traits>(_L), StringBase<T, Traits>::_AppendStub(_R)); }
template<class T, class Traits> inline
	StringAppender<StringWrapper<Traits>, StringBase<T, Traits>::_AppendStub, Traits>
	operator+(typename Traits::const_pointer _L, const StringBase<T, Traits>& _R)
		{ return StringAppender<StringWrapper<Traits>, StringBase<T, Traits>::_AppendStub,
			Traits>(StringWrapper<Traits>(_L), StringBase<T, Traits>::_AppendStub(_R)); }
template<class T, class Traits> inline
	StringAppender<StringBase<T, Traits>::_AppendStub, StringBase<T, Traits>::_AppendStub, Traits>
	operator+(const StringBase<T, Traits>& _L, const StringBase<T, Traits>& _R)
		{ return StringAppender<StringBase<T, Traits>::_AppendStub, StringBase<T, Traits>::_AppendStub,
			Traits>(StringBase<T, Traits>::_AppendStub(_L), StringBase<T, Traits>::_AppendStub(_R)); }
template<class TL, class TR, class Traits> inline
	StringAppender<StringAppender<TL, TR, Traits>, CharWrapper<Traits>, Traits>
	operator+(const StringAppender<TL, TR, Traits>& _L, typename Traits::const_reference _R)
		{ return StringAppender<StringAppender<TL, TR, Traits>,
			CharWrapper<Traits>, Traits>(_L, CharWrapper<Traits>(_R)); }
template<class TL, class TR, class Traits> inline
	StringAppender<StringAppender<TL, TR, Traits>, StringWrapper<Traits>, Traits>
	operator+(const StringAppender<TL, TR, Traits>& _L, typename Traits::const_pointer _R)
		{ return StringAppender<StringAppender<TL, TR, Traits>,
			StringWrapper<Traits>, Traits>(_L, StringWrapper<Traits>(_R)); }
template<class T, class TL, class TR, class Traits> inline
	StringAppender<StringAppender<TL, TR, Traits>, StringBase<T, Traits>::_AppendStub, Traits>
	operator+(const StringAppender<TL, TR, Traits>& _L, const StringBase<T, Traits>& _R)
		{ return StringAppender<StringAppender<TL, TR, Traits>,
			StringBase<T, Traits>::_AppendStub, Traits>(_L, StringBase<T, Traits>::_AppendStub(_R)); }
template<class TL, class TR, class Traits> inline
	StringAppender<CharWrapper<Traits>, StringAppender<TL, TR, Traits>, Traits>
	operator+(typename Traits::const_reference _L, const StringAppender<TL, TR, Traits>& _R)
		{ return StringAppender<CharWrapper<Traits>,
			StringAppender<TL, TR, Traits>, Traits>(CharWrapper<Traits>(_L), _R); }
template<class TL, class TR, class Traits> inline
	StringAppender<StringWrapper<Traits>, StringAppender<TL, TR, Traits>, Traits>
	operator+(typename Traits::const_pointer _L, const StringAppender<TL, TR, Traits>& _R)
		{ return StringAppender<StringWrapper<Traits>,
			StringAppender<TL, TR, Traits>, Traits>(StringWrapper<Traits>(_L), _R); }
template<class T, class TL, class TR, class Traits> inline
	StringAppender<StringBase<T, Traits>::_AppendStub, StringAppender<TL, TR, Traits>, Traits>
		operator+(const StringBase<T, Traits>& _L, const StringAppender<TL, TR, Traits>& _R)
			{ return StringAppender<StringBase<T, Traits>::_AppendStub, StringAppender<TL, TR, Traits>,
				Traits>(StringBase<T, Traits>::_AppendStub(_L), _R); }
template<class TLL, class TLR, class TRL, class TRR, class Traits> inline
	StringAppender<StringAppender<TLL, TLR, Traits>, StringAppender<TRL, TRR, Traits>, Traits>
	operator+(const StringAppender<TLL, TLR, Traits>& _L, const StringAppender<TRL, TRR, Traits>& _R)
		{ return StringAppender<StringAppender<TLL, TLR, Traits>,
			StringAppender<TRL, TRR, Traits>, Traits>(_L, _R); }

typedef StringBase<char> String;
typedef StringBase<wchar_t> WString;

#pragma pack(pop)
